package cic.wsd.disambiguation;

import java.util.ArrayList;

import cic.wordnet.ParsedSynset;
import cic.wordnet.WordNet;
import cic.wordnet.pairs.KeyString;
import cic.wsd.semcor.AmbiguousWord;
import cic.wsd.semcor.Input;
import cic.wsd.testing.Decision;
import cic.wsd.windowing.Condition;
/**
 * Generic template for a bag-of-words disambiguation algorithm.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public abstract class WSDAlgorithm {
	/**
	 * ArrayList for containing extra parameters and its values.
	 */
	ArrayList<KeyString> params;
	
	/**
	 * Tells the disambiguation algorithm to solve an input document.
	 * @param document The disambiguated document.
	 * @param conditions Conditions for window selection.
	 * @param windowSize Maximum number of words in the window.
	 * @param backoff Back-off strategy.
	 * @param tie Algorithm to be used for solving ties.
	 * @return Decisions made by the algorithm.
	 */
	public ArrayList<Decision> solve(Input document, ArrayList<Condition> conditions,int windowSize, WSDAlgorithm backoff, WSDAlgorithm tie)
	{
		ArrayList<Decision> decisions=new ArrayList<Decision>(document.getAmbiguousWords().size());
		int target=0;
		for(AmbiguousWord targetWord:document.getAmbiguousWords())
		{
			ArrayList<AmbiguousWord> window= new ArrayList<AmbiguousWord>(windowSize);
			int i=1;
			// Retrieve window
			while((window.size()<windowSize)&&((target-i>=0)||(target+i<document.getAmbiguousWords().size())))
			{
				if(target-i>=0)
				{
					AmbiguousWord word=document.getAmbiguousWords().get(target-i);
					boolean ban=true;
					for(Condition condition:conditions)
					{
						if(condition.satisfiesCondition(targetWord, word, window)==false)
							{
								ban=false;
								break;
							}
					}
					if(ban)
					{
						window.add(word);
					}
				}
				if(window.size()==windowSize)
					break;
				if(target+i<document.getAmbiguousWords().size())
				{
					AmbiguousWord word=document.getAmbiguousWords().get(target+i);
					boolean ban=true;
					for(Condition condition:conditions)
					{
						ban=ban&&condition.satisfiesCondition(targetWord, word, window);
					}
					if(ban)
					{
						window.add(word);
					}
				}
				i++;
			}
			Decision d=this.disambiguate(targetWord, window);
			if(d.isAttempted())
			{
				if(d.isTied()&&tie!=null)
				{
					d.unTie(tie.disambiguate(targetWord, window));
				}				
			}
			else
			{
				if(backoff!=null)
				{
					d=backoff.disambiguate(targetWord, window);
				}
			}
			decisions.add(d);
			target++;
			
		}
		return decisions;
	}
	
	/**
	 * Disambiguates target using the window.
	 * @param target The AmbiguousWord to be disambiguated.
	 * @param window The context window that helps disambiguation process.
	 * @return The Decision made by the algorithm.
	 */
	public abstract Decision disambiguate(AmbiguousWord target,ArrayList<AmbiguousWord> window);
	/**
	 * Return the name of the algorithm.
	 */
	public abstract String toString();
	/**
	 * Tells if the overlap between a sense and a lemma exists.
	 * @param sense Target sense.
	 * @param lemma Target lemma.
	 * @return true if there is an overlap.
	 */
	public boolean overlap(ParsedSynset sense,String lemma)
	{
		for(String word:sense.getBagOfWords())
			if(word.equals(lemma))
					return true;
  		return false;
	}
	
	/**
	 * Returns the overlap between two senses.
	 * @param sense Sense 1.
	 * @param sense2 Sense 2.
	 * @param dwords ArrayList for storing the overlapping words.
	 * @return true if there is an overlap.
	 */
	public boolean overlap(ParsedSynset sense,ParsedSynset sense2,ArrayList<String> dwords)
	{
		boolean ban=false;
  		for(String oword:sense2.getBagOfWords())
  		{
  			if(WordNet.getIDF(oword)>=3.5)
  			if(this.overlap(sense, oword))
  			{
  				dwords.add(oword);
  				ban=true;
  			}
  		}
  		return ban;
	}
	
	/**
	 * Returns the overlap between a sense and an AmbiguousWord.
	 * @param sense Target sense.
	 * @param lemma Target AmbiguousWord.
	 * @param dwords ArrayList for storing the overlapping words.
	 * @return true if there is an overlap.
	 */
	public boolean overlap(ParsedSynset sense,AmbiguousWord lemma,ArrayList<String> dwords)
	{
		boolean ban=false;
  		for(ParsedSynset nsense:lemma.getSenses())
  			if(this.overlap(sense, nsense, dwords))
  				ban=true;
  		return ban;
	}
	/**
	 * Method for setting extra parameters.
	 * @param string String containing the values and names of the extra parameters.
	 */
	public void setParams(String string) {
		String aux[]=string.split(",");
		for(String param:aux)
		{
			String x[]=param.split(":");
			this.params.add(new KeyString(x[0],x[1]));
		}

		this.params=new ArrayList<KeyString>(aux.length);
		
	}
	/**
	 * 
	 * @return An String like "((param1:value1) , ... , (paramN:valueN))".
	 */
	public String getParams()
	{
		String x="( ";
		if(this.params!=null)
			for(KeyString aux:this.params)
			{
				x+=aux.toString()+" , ";
			}
		x=x.substring(0,x.length()-1)+")";
		return x;
	}
}
