package cic.wsd.semcor;

import java.util.ArrayList;

import cic.wordnet.ParsedSynset;
import cic.wordnet.WordNet;
/**
 * Class for storing an ambiguous lemma.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public class AmbiguousWord {
	/**
	 * WordNet lemma retrieved from WordNet.Morphy() method.
	 */
	String lemma;
	/**
	 * POS tag of the lemma.
	 */
	String pos;
	/**
	 * Senses for the lemma retrieved from WordNet using getLemma(lemma).
	 */
	ArrayList<ParsedSynset> senses;
	/**
	 * Senses retrieved from SEMCOR wnsn attribute.
	 */
	int []correctSenses;
	/**
	 * IDF from the lemma retrieved from WordNet.getIDF(lemma)
	 */
	double idf;
	/**
	 * TF value of this lemma set by an Input object.
	 */
	double tf;
	
	/**
	 * The index of this word in the current document.
	 */
	
	int index;
	/**
	 * Return the position of this word in the document.
	 * @return index
	 */
	public int getIndex()
	{
		return this.index;
	}
	/**
	 * Returns the lemma.
	 */
	public String toString()
	{
		return getLemma();
	}
	/**
	 * Return the TF value of this word.
	 * @return tf
	 */
	public double getTf() {
		return tf;
	}
	/**
	 * Sets the TF value of this lemma in its current document.
	 * @param tf TF value.
	 */
	public void setTf(double tf) {
		this.tf = tf;
	}
	/**
	 * Returns the IDF value of this lemma.
	 * @return idf
	 */
	public double getIDF()
	{
		return idf;
	}
	
	/**
	 * Returns this lemma in format "lemma_P".
	 * @return Lemma in format "lemma_P".
	 */
	public String getLemma() {
		
		return lemma+"_"+this.pos;
	}
	/**
	 * Returns this word's POS tag.
	 * @return POS tag
	 */
	public String getPOS() {
		return this.pos;
	}
	/**
	 * Returns the possible senses of this lemma.
	 * @return Possible senses of this lemma.
	 */
	public ArrayList<ParsedSynset> getSenses() 
	{
		return senses;
	}
	/**
	 * Returns the correct senses of this word.
	 * @return correctSenses
	 */
	public int[] getCorrectSenses() {
		return correctSenses;
	}
	/**
	 * Creates an ambiguous word from SEMCOR info.
	 * @param lemma Value of SEMCOR lemma attribute.
	 * @param pos Value of SEMCOR pos attribute.
	 * @param correctSenses Value of SEMCOR wnsn attribute. 
	 * "U" means that any sense is correct.
	 * @param index The problem number.
	 */
	public AmbiguousWord(String lemma, String pos,
			String correctSenses, int index)
	{
		this.index=index;
		this.lemma=WordNet.Morphy(lemma, pos).get(0);
		this.pos=pos;
		if(this.pos.equals("W"))
			this.pos="R";
		String split[]=correctSenses.split(";");
		this.correctSenses=new int[split.length];
		this.idf=WordNet.getIDF(this.getLemma());
		int i;
		
		this.senses=WordNet.getLemma(this.lemma+"_"+this.pos);
		if(!split[0].equals("U"))
		{
			i=0;
			for(String answer:split)
			{			
				this.correctSenses[i]=Integer.parseInt(answer)-1;
				i++;
			}
		}
		else
		{
			for(i=0;i<this.senses.size();i++)
				this.correctSenses[i]=i;
		}
	}
	/**
	 * Returns an ArrayList containing the possible synonyms of this lemma. 
	 * Synonyms are retrieved from all the senses of this word.
	 * @return An ArrayList containing the possible synonyms of this lemma.
	 */
	public ArrayList<String> getPossibleSynonyms() {
		ArrayList<String> synonyms=new ArrayList<String>();
		for(ParsedSynset ps:this.senses)
			synonyms.addAll(ps.getSynonyms());
		return Util.removeDuplicates(synonyms);
	}
	/**
	 * Returns an ArrayList containing the possible hypernyms of this lemma.
	 * Hyperyms are retrieved from all the senses of this word.
	 * @return An ArrayList containing the possible hypernyms of this lemma.
	 */
	public ArrayList<String> getPossibleHypernyms() {
		ArrayList<String> hypernyms= new ArrayList<String>();
		for(ParsedSynset ps:this.senses)
		{
			ArrayList<String> hyps=ps.getSyds("@");	
			hyps.addAll(ps.getSyds("@i"));
			for(String hyp:hyps)
			{
				hypernyms.addAll(WordNet.getSynset(hyp).getSynonyms());
			}
		}
		return Util.removeDuplicates(hypernyms);
	}
	
}
