package cic.wsd.testing;

import java.util.ArrayList;


import cic.wsd.semcor.AmbiguousWord;

/**
 * Class for storing a decision made by the disambiguation algorithm.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public class Decision {
  /**
	* Senses selected by the algorithm.
	*/
  int []answers;
  /**
   * The disambiguated word.
   */
  AmbiguousWord target;
  /**
   * Context words used for disambiguating a word.
   */
  ArrayList<AmbiguousWord> window;
  /**
   * Array containing the words used for scoring each sense of the target word.
   */
  ArrayList<ArrayList<String>>decisionWords;
  /**
   * Array containing sense weights given by the disambiguation algorithm. 
   */
  double []weights;
  /**
   * True, if the algorithms sets at least a single weight.
   */
  boolean attempted;
  /**
   * Sets a sense weight and the words used for scoring the sense.
   * @param sense Measured sense.
   * @param w Weight given by the disambiguation algorithm.
   * @param words The words used for scoring this sense. words must be an empty ArrayList if w=0.
   */
  public void setSense(int sense,double w, ArrayList<String> words)
  {
	  this.weights[sense]=w;
	  this.decisionWords.get(sense).addAll(words);
  }
  /**
   * Returns the words used for scoring a sense.
   * @param sense Target sense number.
   * @return The words used for scoring a sense.
   */
  public ArrayList<String> getDecisionWords(int sense)
  {
	  return decisionWords.get(sense);
  }

  /**
   * Returns true, if the algorithms sets at least a single weight.
   * @return True, if the algorithms sets at least a single weight.
   */
  public boolean isAttempted()
  {
	  return attempted;
  }

  /**
   * This method is the one who sets the system answers. 
   * This method picks the sense(s) with the highest weight.
   */
  public void calculateAnswer()
  {
	  int bestScore=0;
	  int i=0;
	  //Searching for the best score
	  for(double w:this.weights)
	  {
		  if(w>this.weights[bestScore])
			  bestScore=i;
		  i++;
	  }
	  //Looking for ties
	  int count=0;
	  if(this.weights[bestScore]>0.0)
		  for(double w:this.weights)
			  if(w==this.weights[bestScore])
				  count++;
	  
	  this.answers=new int[count];
	  this.attempted=false;
	  if(this.weights.length>0)
	  {
		if(this.weights[bestScore]>0.0)
		{
			this.attempted=true;
			i=0;
			int z=0;
			//getting correct answers
			for(double w:this.weights)
			{
				  if(w==this.weights[bestScore])
				  {
					  this.answers[z]=i;
					  z++;
				  }
				  i++;
			}
		}
	  }
  }
/**
 * Prepares a new decision with empty weights.
 * @param target The disambiguated word.
 * @param window Words used for disambiguating the target.
 */
public Decision(AmbiguousWord target, ArrayList<AmbiguousWord> window) {
	super();
	this.target = target;
	this.window = window;
	this.weights=new double[target.getSenses().size()];
	this.decisionWords=new ArrayList<ArrayList<String>>(this.weights.length);
	for(int i=0;i<this.weights.length;i++)
	{
		this.weights[i]=0.0d;
		this.decisionWords.add(new ArrayList<String>());
	}
}


/**
 * Returns the answers of the algorithm.
 * @return answers
 */
public int []getAnswers() {
	return answers;
}
/**
 * Returns the target word.
 * @return target
 */
public AmbiguousWord getTarget() {
	return target;
}
/**
 * Returns the window.
 * @return window
 */
public ArrayList<AmbiguousWord> getWindow() {
	return window;
}
/**
 * Returns the scores of each sense.
 * @return weights
 */
public double[] getWeights() {
	return weights;
}
/**
 * Returns "[windowWord1,...,windowWordN]".
 * @return "[windowWord1,...,windowWordN]"
 */
public String getWindowPrint() {
	String Print="[";
	for(AmbiguousWord word:this.window)
		Print+=" "+word.toString();
	Print+="]";
	return Print;
}

/**
 * Returns the number of senses disambiguated.
 * @return The number of senses disambiguated.
 */
public int getSenseCount()
{
	return this.weights.length;
}
/**
 * Returns a count of the number of senses with a score>0
 * @return The count of the number of addressed senses
 */
public int getSensesAddressedCount()
{
	int i=0;
	for(double d:this.weights)
	{
		if(d>0.0d)
			i++;
	}
	return i;
}

/**
 * Returns true, if an algorithm answer are among the correct answers.
 * @return True, if an algorithm answer are among the correct answers.
 */
public boolean isCorrectSenseAddressed()
{
	for(int i:target.getCorrectSenses())
	{
		if(i>=0&&i<this.weights.length)
			if(this.weights[i]>0.0d)
				return true;
	}
	return false;
}
/**
 * Returns true, if the algorithm returned more than one correct answer.
 * @return True, if the algorithm returned more than one correct answer.
 */
public boolean isTied()
{
  	return this.answers.length>1;
}
/**
 * Solves a tie with the answer provided by another disambiguation approach.
 * @param tie answer provided by another disambiguation approach.
 */
public void unTie(Decision tie) {
	for(int answer:this.answers)
	{
		this.weights[answer]+=tie.getWeights()[answer];
	}
	this.calculateAnswer();
}
/**
 * Calculates this decision's score following the Senseval score system.
 * @return The score obtained by the system answers (it ranges in [0.0,1.0]).
 */
public double getScore()
{
	double score=0.0;
	for(int answer:this.answers)
	{
		for(int correct:this.target.getCorrectSenses())
		{
			if(answer==correct)
			{
				score+=1.0/((double)this.answers.length);
			}
		}
	}
	return score;
}
/**
 * Returns true if answer is among the correct senses.
 * @param answer Answer to be checked.
 * @return True if answer is among the correct senses.
 */
public boolean isCorrect(int answer) {
	for(int correct:this.target.getCorrectSenses())
		if(answer==correct)
			return true;
	return false;
}
/**
 * Returns "( [answer1 [,answerN]*]* )".
 * @return "( [answer1 [,answerN]*]* )".
 */
public String getAnswersString() {
	String answers="( ";
	for(int answer:this.answers)
		answers+=" "+answer + ",";
	answers=answers.substring(0,answers.length()-1)+" )";
	return answers;
}

}