package cic.wsd.testing;

import java.util.ArrayList;

import cic.wsd.disambiguation.WSDAlgorithm;
import cic.wsd.semcor.Input;
import cic.wsd.windowing.Condition;
/**
 * Class for setting a test of a disambiguation method over a set of SEMCOR documents.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public class Test {
	/**
	 * The set of SEMCOR documents used as test-bed.
	 */
	ArrayList<Input> docs;
	/**
	 * Summaries of the test results obtained in each document.
	 */
	ArrayList<Summary> summaries;
	/**
	 * Answers obtained in each document.
	 */
	ArrayList<ArrayList<Decision>> answers;

	/**
	 * Algorithm to be tested.
	 */
	WSDAlgorithm algorithm;
	/**
	 * Window size to be used by the windowing method. 
	 */
	int windowSize;
	/**
	 * Disambiguation algorithm used as a back-off strategy. 
	 * Use null to indicate no back-off.
	 */
	WSDAlgorithm backoff;
	/**
	 * Name of the folder/file containing the tests
	 */
	String testSet;
	/**
	 * Disambiguation algorithm used for solving ties. 
	 * Use null for leaving the ties unsolved.
	 */
	WSDAlgorithm tie;
	/**
	 * Conditions for retrieving the window.
	 */
	ArrayList<Condition> conditions;
	/**
	 * Pruning method used.
	 */
	String retrievedSenses;
	/**
	 * Samples loaded.
	 */
	String KNSources;
	/**
	 * Returns the test-bed used on this test.
	 * @return docs
	 */
	public ArrayList<Input> getDocs() {
		return docs;
	}
	/**
	 * Returns the summaries of the test runs.
	 * @return summaries
	 */
	public ArrayList<Summary> getSummaries() {
		return summaries;
	}
	/**
	 * Returns the decisions made on the test runs.
	 * @return answers
	 */
	public ArrayList<ArrayList<Decision>> getAnswers() {
		return answers;
	}
	
	
	/**
	 * Returns the disambiguation algorithm used on this test.
	 * @return algorithm
	 */
	public WSDAlgorithm getAlgorithm() {
		return algorithm;
	}
	/**
	 * Returns the back-off strategy used on this test.
	 * @return backoff
	 */
	public WSDAlgorithm getBackoff() {
		return backoff;
	}
	/**
	 * Returns the window size used on this test.
	 * @return windowSize
	 */
	public int getWindowSize() {
		return windowSize;
	}


	/**
	 * Run the current test.
	 * @return The decisions made by the disambiguation method over the test-bed.
	 */
	public ArrayList<ArrayList<Decision>> run()
	{
		System.out.println("Test started for algorithm "+this.algorithm
							+ " with backoff "+this.backoff
							+ " and  over "+this.testSet);
		this.answers=new ArrayList<ArrayList<Decision>>(this.docs.size());
		this.summaries=new ArrayList<Summary>(this.docs.size());
		for(Input doc:docs)
		{
			ArrayList<Decision> decisions=this.algorithm.solve(doc, this.conditions, windowSize, backoff, tie);
			this.answers.add(decisions);
			this.summaries.add(new Summary(decisions));
		}
		System.out.println("Finished \t p:"+String.valueOf(Measures.Precision(this.summaries)[4])+" r:"+String.valueOf(Measures.Recall(summaries)[4])+" f:"+String.valueOf(Measures.Fmeasure(summaries)[4]));
		return this.answers;
	}
	/**
	 * Returns the sample sets loaded.
	 * @return KNSources
	 */
	public String getKNSources() {
		return this.KNSources;
	}
	/**
	 * Returns the pruning method used.
	 * @return retrievedSenses
	 */
	public String getRetrievedSenses() {
		return this.retrievedSenses;
	}
	/**
	 * Returns the algorithm used for solving ties.
	 * @return tie
	 */
	public WSDAlgorithm getTie() {
		return this.tie;
	}
	/**
	 * Returns the conditions for selecting the window words.
	 * @return conditions
	 */
	public ArrayList<Condition> getConditions() {
		return this.conditions;
	}
	/**
	 * Creates a new test.
	 * @param docs The SEMCOR documents used as test-bed.
	 * @param algorithm Disambiguation method to test to.
	 * @param windowSize Window size to be used by the windowing method.
	 * @param backoff Disambiguation algorithm to be used as a back-off strategy. Use null to indicate no back-off.
	 * @param testSet Name of the folder/file containing the tests.
	 * @param tie Tie solving strategy.
	 * @param conditions Conditions for retrieving window words from context.
	 * @param retrievedSenses Pruning method used.
	 * @param KNSources Loaded samples.
	 */
	
	public Test(ArrayList<Input> docs, WSDAlgorithm algorithm,
			int windowSize, WSDAlgorithm backoff, String testSet,
			WSDAlgorithm tie, ArrayList<Condition> conditions,
			String retrievedSenses, String KNSources) {
		super();
		this.docs = docs;
		this.algorithm = algorithm;
		this.windowSize = windowSize;
		this.backoff = backoff;
		this.testSet = testSet;
		this.tie = tie;
		this.conditions = conditions;
		this.retrievedSenses = retrievedSenses;
		this.KNSources = KNSources;
	}
	
}
