package cic.wsd.testing;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map.Entry;

import jxl.Workbook;
import jxl.write.Label;
import jxl.write.Number;
import jxl.write.NumberFormat;
import jxl.write.WritableCellFormat;
import jxl.write.WritableFont;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import cic.wordnet.WordNet;
import cic.wsd.semcor.Input;
import cic.wsd.semcor.Util;
import cic.wsd.windowing.Condition;
/**
 * Class for writing the test result in XLS format.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public final class XLSWriter {
	static WritableCellFormat number = new WritableCellFormat(new NumberFormat("###.##"));
	static WritableCellFormat boldnumber= new WritableCellFormat(new WritableFont(WritableFont.ARIAL, 11, WritableFont.BOLD, true),new NumberFormat("###.##")); 
	/**
	 * Writes an XLS file.
	 * @param file File to be created.
	 * @param tests The tests to be included in the file.
	 * @param detailed Tells if a decisions detail sheet will be added to the document.
	 * @throws Exception
	 */
	public static void writeXLS(File file, ArrayList<Test> tests, boolean detailed) throws Exception
	{
		System.out.println("Saving "+file.getName());
		WritableWorkbook workbook = Workbook.createWorkbook(file);
		WritableSheet header = workbook.createSheet("Experimental setup", 0);
		WritableSheet performance = workbook.createSheet("Performance", 1);
		WritableSheet summary = workbook.createSheet("Decisions summary", 2);
		WritableSheet docs = workbook.createSheet("Problem summary", 3);
		WritableSheet misc = workbook.createSheet("Miscelaneous", 4);
		if(detailed)
		{
			WritableSheet detail = workbook.createSheet("Decisions detail", 5);
			addDecisionsDetailSheet(detail,tests);
			
		}
		addHeader(header,tests);
		addPerformanceSheet(performance,tests);
		addDecisionsSumSheet(summary,tests);
		addProblemSummary(docs,tests);
		addMiscSheet(misc, tests);
		workbook.write();
		workbook.close();
	}
	/**
	 * Sheet describing the file contents.
	 * @param sheet The sheet to be filled.
	 * @param tests Tests included in this file.
	 * @throws Exception
	 */
	private static void addHeader(WritableSheet sheet, ArrayList<Test> tests)throws Exception
	{
		Test sample=tests.get(0);
		sheet.addCell(new Label(0, 0, "CICNLP output file. Created by: Viveros-Jimenez F., Gelbukh A. & Sidorov G."));
		sheet.addCell(new Label(0, 2, "Knowledge Source:")); 
		sheet.addCell(new Label(1, 2, sample.getKNSources()));
		sheet.addCell(new Label(1, 3, "* Retrieved Senses:")); 
		sheet.addCell(new Label(2, 3, sample.getRetrievedSenses()));
		sheet.addCell(new Label(0, 5, "Tests"));
		int offset=6;
		int i=1;
		for(Test test:tests)
		{
			sheet.addCell(new Label(0, offset, "Test "+i));
			offset++;
			sheet.addCell(new Label(1, offset, "* WSD method:"));
			sheet.addCell(new Label(2, offset, test.getAlgorithm().toString()));
			offset++;
			sheet.addCell(new Label(1, offset, "* Back-off method:"));
			if(test.getBackoff()!=null)
				sheet.addCell(new Label(2, offset, test.getBackoff().toString()));
			else
				sheet.addCell(new Label(2, offset, "none"));
			offset++;
			sheet.addCell(new Label(1, offset, "* Tie solving method:"));
			if(test.getTie()!=null)
				sheet.addCell(new Label(2, offset, test.getTie().toString()));
			else
				sheet.addCell(new Label(2, offset, "none"));
			offset++;
			sheet.addCell(new Label(1, offset, "* Window size:"));
			sheet.addCell(new Label(2, offset, String.valueOf(test.getWindowSize())));
			offset++;
			sheet.addCell(new Label(1, offset, "* Windowing Conditions:"));
			offset++;
			for(Condition condition:test.getConditions())
			{
				sheet.addCell(new Label(2, offset, condition.toString()));
				offset++;
			}
			offset++;
			i++;
		}
		
	}
	
	/**
	 * Writes a decision detail sheet. 
	 * This sheet contains a more detailed explanation of how decisions were made.
	 * @param sheet The sheet to be created.
	 * @param tests Data for creating the sheet.
	 * @throws Exception
	 */
	private static void addDecisionsDetailSheet(WritableSheet sheet, ArrayList<Test> tests)throws Exception
	{
	 
		 
		int goffset=0;
		int t=1;
		for(Test test:tests)
		{
			int offset=4;
			sheet.addCell(new Label(goffset, offset, "Tests "+t)); 
			offset++;	
			t++;
			for(int i=0;i<test.getDocs().size();i++)
			{
				Input doc=test.getDocs().get(i);
				ArrayList<Decision> decisions=test.getAnswers().get(i);
				sheet.addCell(new Label(goffset, offset, "Document:")); 
				sheet.addCell(new Label(goffset+1, offset, doc.toString())); 
				offset++;
				
				for(Decision decision:decisions)
				{
					if(decision.isAttempted())
					{
						sheet.addCell(new Label(goffset+1,offset,"Lemma"));
						sheet.addCell(new Label(goffset+2,offset,"Polysemy"));
						sheet.addCell(new Label(goffset+3,offset,"Correct Answers"));
						sheet.addCell(new Label(goffset+4,offset,"Approach Answer"));
						sheet.addCell(new Label(goffset+5,offset,"Window"));
						offset++;
						sheet.addCell(new Label(goffset+1,offset,decision.getTarget().getLemma()));
						sheet.addCell(new Number(goffset+2,offset,decision.getTarget().getSenses().size()));
						sheet.addCell(new Label(goffset+3,offset,Arrays.toString(decision.getTarget().getCorrectSenses())));
						sheet.addCell(new Label(goffset+4,offset,decision.getAnswersString()));
						sheet.addCell(new Label(goffset+5,offset,decision.getWindowPrint()));
						offset++;
						sheet.addCell(new Label(goffset+1,offset,"Sense"));
						sheet.addCell(new Label(goffset+2,offset,"Weight"));
						sheet.addCell(new Label(goffset+3,offset,"Words"));
						offset++;
						int j=0;
						for(double w:decision.getWeights())
						{
							if(w>0.0)
							{
								sheet.addCell(new Number(goffset+1,offset,j+1));
								sheet.addCell(new Number(goffset+2,offset,w));
								sheet.addCell(new Label(goffset+3,offset,decision.getDecisionWords(j).toString()));
								
								offset++;
							}
							j++;
						}
						
						offset++;
						offset++;
					}
				 
				}
				
				
				offset++;
				i++;
			}
			
			offset++;
			goffset+=6;
		}
	}
	/**
	 * Writes a summary of the decisions made.
	 * @param sheet The sheet to write to.
	 * @param tests Data for creating the sheet.
	 * @throws Exception
	 */
	private static void addDecisionsSumSheet(WritableSheet sheet, ArrayList<Test> tests) throws Exception
	{
		
		
		
		int goffset=4;
		int t=1;
		for(Test test:tests)
		{
			HashMap<String,int[]> globalLemmas=new HashMap<String,int []>();	
			int i=0;
			int docs=test.getSummaries().size();
			for(Summary summary:test.getSummaries())
			{
				for(Entry<String,int[]> entry:summary.getDecisionLemmas().entrySet())
				{
					
					if(globalLemmas.containsKey(entry.getKey()))
					{
						int []count=globalLemmas.get(entry.getKey());
						int []aux=entry.getValue();
						count[i]+=aux[0];
						count[docs]+=aux[0];
						count[docs+1]+=aux[1];
					}
					else
					{
						int []count=new int[docs+2];
						for(int j=0;j<(docs+1);j++)
							count[j]=0;
						int []aux=entry.getValue();
						count[i]+=aux[0];
						count[docs]+=aux[0];
						count[docs+1]+=aux[1];
						globalLemmas.put(entry.getKey(), count);
					}
				}
				i++;
			}
			//write the header parameters of the test
			int offset=0;
			sheet.addCell(new Label(0, goffset, "Test "+t));
			t++;
			goffset++;
			sheet.addCell(new Label(0, goffset, "Lemma")); 
				
			for(Input doc:test.getDocs())
			{
				sheet.addCell(new Label(1+offset, goffset, doc.toString())); 
				offset++;
			}
			sheet.addCell(new Label(1+offset, goffset, "Overall attempts"));
			sheet.addCell(new Label(2+offset, goffset, "Overall correct answers"));
			sheet.addCell(new Label(3+offset, goffset, "IDF"));
			goffset++;
			for(Entry<String,int[]> entry:globalLemmas.entrySet())
			{
				int[] counts=entry.getValue();
				sheet.addCell(new Label(0,goffset,entry.getKey()));
				for(int z=0;z<counts.length;z++)
				{
					if(z<counts.length-2)
						sheet.addCell(new Number(1+z, goffset, counts[z],number));
					else
						sheet.addCell(new Number(1+z, goffset, counts[z],boldnumber));
				}
				sheet.addCell(new Number(1+counts.length, goffset, WordNet.getIDF(entry.getKey()),boldnumber));
				goffset++;
			}
			goffset++;
			goffset++;
			goffset++;
		}
		
	}
	/**
	 * Generates a summary of the lemmas on the test-bed
	 * @param sheet The sheet to write to.
	 * @param tests Data for creating the sheet.
	 * @throws Exception
	 */
	private static void addProblemSummary(WritableSheet sheet, ArrayList<Test> tests) throws Exception
	{
		//Extract the decisions detail
		//Header

		
		HashMap<String,int[]> globalLemmas=new HashMap<String,int []>();
		int goffset=3;
		int i=0;
		int docs=tests.get(0).getDocs().size();
		for(Input doc:tests.get(0).getDocs())
		{
				for(Entry<String,ArrayList<Integer>> entry:doc.getIndex().entrySet())
				{
					
					if(globalLemmas.containsKey(entry.getKey()))
					{
						int []count=globalLemmas.get(entry.getKey());
						count[i]+=entry.getValue().size();
						count[docs]+=entry.getValue().size();
					}
					else
					{
						int []count=new int[docs+1];
						for(int j=0;j<(docs+1);j++)
							count[j]=0;
						count[i]+=entry.getValue().size();
						count[docs]+=entry.getValue().size();
						globalLemmas.put(entry.getKey(), count);
					}
				}
				i++;
		}
		sheet.addCell(new Label(0, goffset, "Lemma")); 
		int offset=0;
		for(Input doc:tests.get(0).getDocs())
		{
			sheet.addCell(new Label(1+offset, goffset, doc.toString())); 
			offset++;
		}
		sheet.addCell(new Label(1+offset, goffset, "Overall appareances"));
		sheet.addCell(new Label(2+offset, goffset, "IDF"));
		goffset++;
		for(Entry<String,int[]> entry:globalLemmas.entrySet())
		{
			int[] counts=entry.getValue();
			sheet.addCell(new Label(0,goffset,entry.getKey()));
			int z;
			for(z=0;z<counts.length;z++)
			{
					if(z==(counts.length-1))
						sheet.addCell(new Number(1+z, goffset, counts[z],boldnumber));
					else
						sheet.addCell(new Number(1+z, goffset, counts[z],number));
					
			}
			sheet.addCell(new Number(1+z,goffset,WordNet.getIDF(entry.getKey()),boldnumber));
			goffset++;
		}
		
	}
	/**
	 * Writes a summary of the performance of the algorithm.
	 * @param sheet The sheet to write to.
	 * @param tests Data for creating the sheet.
	 * @throws Exception
	 */
	private static void addPerformanceSheet(WritableSheet sheet, ArrayList<Test> tests) throws Exception
	{
		int offset=3;
		int t=1;
		for(Test test:tests)
		{
			sheet.addCell(new Label(0, offset, "Test "+t)); 
			t++;
			sheet.addCell(new Label(2, offset, "Nouns")); 
			sheet.addCell(new Label(5, offset, "Verbs"));
			sheet.addCell(new Label(8, offset, "Adjectives"));
			sheet.addCell(new Label(11, offset, "Adverbs"));
			sheet.addCell(new Label(14, offset, "Overall"));
			offset++;
			for(int i=0;i<5;i++)
			{
				sheet.addCell(new Label(i*3+1, offset, "Precision"));
				sheet.addCell(new Label(i*3+2, offset, "Recall"));
				sheet.addCell(new Label(i*3+3, offset, "F1-measure"));
			}
			offset++;
			ArrayList<Summary> global=new ArrayList<Summary>();
			int d=0;
			for(Input doc:test.getDocs())
			{
				sheet.addCell(new Label(0, offset, doc.toString()));
				Summary summary=test.getSummaries().get(d);
				double[]p=Measures.Precision(summary);
				double[]r=Measures.Recall(summary);
				double[]f=Measures.Fmeasure(summary);
				for(int j=0;j<5;j++)
				{
					if(j==4)
					{
						sheet.addCell(new Number(j*3+1, offset, p[j],boldnumber));
						sheet.addCell(new Number(j*3+2, offset, r[j],boldnumber));
						sheet.addCell(new Number(j*3+3, offset, f[j],boldnumber));
					}
					else
					{
						sheet.addCell(new Number(j*3+1, offset, p[j],number));
						sheet.addCell(new Number(j*3+2, offset, r[j],number));
						sheet.addCell(new Number(j*3+3, offset, f[j],number));
					}
				}
				global.add(summary);
				d++;
				offset++;
			}
			sheet.addCell(new Label(0,offset,"Overall:"));
			double[]p=Measures.Precision(global);
			double[]r=Measures.Recall(global);
			double[]f=Measures.Fmeasure(global);
			for(int j=0;j<5;j++)
			{
				sheet.addCell(new Number(j*3+1, offset, p[j],boldnumber));
				sheet.addCell(new Number(j*3+2, offset, r[j],boldnumber));
				sheet.addCell(new Number(j*3+3, offset, f[j],boldnumber));
			}
			offset++;
			offset++;
			offset++;
		}
	}
/**
 * Adds a sheet with some interesting summaries.
 * @param sheet The sheet to be filled.
 * @param tests The tests to be included on this file.
 * @throws Exception
 */
	private static void addMiscSheet(WritableSheet sheet, ArrayList<Test> tests) throws Exception
	{
		
		//Document summary
		int offset=3;
		int t=1;
		for(Test test:tests)
		{
			sheet.addCell(new Label(0, offset, "Test "+t));
			t++;
			sheet.addCell(new Label(1, offset, "Average words used")); 
			sheet.addCell(new Label(2, offset, "Average senses addressed"));
			sheet.addCell(new Label(3, offset, "Probability of addresing the correct sense"));
			sheet.addCell(new Label(4, offset, "Average polisemy"));
			sheet.addCell(new Label(5, offset, "Average score"));
			offset++;
			int i=0;
			double total=0.0d;
			ArrayList<ArrayList<Decision>> ds=new ArrayList<ArrayList<Decision>>(test.getAnswers().size());
			
			for(ArrayList<Decision> decisions:test.getAnswers())
			{
				ArrayList<Decision> d=new ArrayList<Decision>(decisions.size());
				for(Decision decision:decisions)
				{
					if(decision.isAttempted())
					{
						d.add(decision);
					}
				}
				ds.add(d);
			}
			
			for(ArrayList<Decision> decisions:ds)
			{
				total+=(double)decisions.size();
			}
			double gwords=0.0d;
			double gsenses=0.0d;
			double gcorrect=0.0d;
			double gpolisemy=0.0d;
			double gscore=0.0d;
			
			for(ArrayList<Decision> decisions:ds)
			{
				double words=0.0d;
				double senses=0.0d;
				double correct=0.0d;
				double polisemy=0.0d;
				double score=0.0d;
				sheet.addCell(new Label(0,offset,test.getDocs().get(i).toString()));
				for(Decision decision:decisions)
				{
					words+=((double)Util.removeDuplicates(
							decision.getDecisionWords(decision.getAnswers()[0])
							).size())/((double)decisions.size());
					gwords+=((double)Util.removeDuplicates(
							decision.getDecisionWords(decision.getAnswers()[0])
							).size())/total;
					senses+=(double)decision.getSensesAddressedCount()/((double)decisions.size());
					gsenses+=(double)decision.getSensesAddressedCount()/total;
					if(decision.isAttempted())
					{
						score+=decision.getWeights()[decision.getAnswers()[0]]/((double)decisions.size());
						gscore+=decision.getWeights()[decision.getAnswers()[0]]/total;
					}
					if(decision.isCorrectSenseAddressed())
					{
						correct+=1.0d/((double)decisions.size());
						gcorrect+=1.0d/total;
					}
					polisemy+=(double)decision.getSenseCount()/((double)decisions.size());
					gpolisemy+=(double)decision.getSenseCount()/total;
				}
				sheet.addCell(new Number(1, offset, words,number)); 
				sheet.addCell(new Number(2, offset, senses,number));
				sheet.addCell(new Number(3, offset, correct,number));
				sheet.addCell(new Number(4, offset, polisemy,number));
				sheet.addCell(new Number(5, offset, score,number));
				offset++;
				i++;
			}
			sheet.addCell(new Label(0,offset,"overall"));
			sheet.addCell(new Number(1, offset, gwords,boldnumber)); 
			sheet.addCell(new Number(2, offset, gsenses,boldnumber));
			sheet.addCell(new Number(3, offset, gcorrect,boldnumber));
			sheet.addCell(new Number(4, offset, gpolisemy,boldnumber));
			sheet.addCell(new Number(5, offset, gscore,boldnumber));
			offset++;
			offset++;
		}
	}



}
